@@ -73,9 +73,12 @@ struct _PeasGtkPluginManagerPrivate {
 enum {
   PROP_0,
   PROP_ENGINE,
-  PROP_VIEW
+  PROP_VIEW,
+  N_PROPERTIES
 };
 
+static GParamSpec *properties[N_PROPERTIES] = { NULL };
+
 G_DEFINE_TYPE (PeasGtkPluginManager, peas_gtk_plugin_manager, GTK_TYPE_BOX);
 
 static gboolean
@@ -333,7 +336,11 @@ populate_popup_cb (PeasGtkPluginManagerView *view,
 static void
 peas_gtk_plugin_manager_init (PeasGtkPluginManager *pm)
 {
-  GtkWidget *hbuttonbox;
+  GtkWidget *toolbar;
+  GtkStyleContext *context;
+  GtkToolItem *toolitem;
+  GtkWidget *toolbar_box;
+  GtkWidget *item_box;
 
   pm->priv = G_TYPE_INSTANCE_GET_PRIVATE (pm,
                                           PEAS_GTK_TYPE_PLUGIN_MANAGER,
@@ -344,7 +351,6 @@ peas_gtk_plugin_manager_init (PeasGtkPluginManager *pm)
   g_irepository_require (g_irepository_get_default (),
                          "PeasGtk", "1.0", 0, NULL);
 
-  gtk_box_set_spacing (GTK_BOX (pm), 6);
   gtk_orientable_set_orientation (GTK_ORIENTABLE (pm),
                                   GTK_ORIENTATION_VERTICAL);
 
@@ -355,18 +361,40 @@ peas_gtk_plugin_manager_init (PeasGtkPluginManager *pm)
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (pm->priv->sw),
                                        GTK_SHADOW_IN);
+  context = gtk_widget_get_style_context (pm->priv->sw);
+  gtk_style_context_set_junction_sides (context, GTK_JUNCTION_BOTTOM);
   gtk_box_pack_start (GTK_BOX (pm), pm->priv->sw, TRUE, TRUE, 0);
 
-  hbuttonbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-  gtk_box_set_spacing (GTK_BOX (hbuttonbox), 6);
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
-  gtk_box_pack_start (GTK_BOX (pm), hbuttonbox, FALSE, FALSE, 0);
+  toolbar = gtk_toolbar_new();
+  gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_MENU);
+  context = gtk_widget_get_style_context (toolbar);
+  gtk_style_context_set_junction_sides (context, GTK_JUNCTION_TOP);
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_INLINE_TOOLBAR);
+  gtk_box_pack_start (GTK_BOX (pm), toolbar, FALSE, FALSE, 0);
 
-  pm->priv->about_button = gtk_button_new_from_stock (GTK_STOCK_ABOUT);
-  gtk_container_add (GTK_CONTAINER (hbuttonbox), pm->priv->about_button);
+  toolitem = gtk_tool_item_new ();
+  gtk_tool_item_set_expand (toolitem, TRUE);
 
-  pm->priv->configure_button = gtk_button_new_from_stock (GTK_STOCK_PREFERENCES);
-  gtk_container_add (GTK_CONTAINER (hbuttonbox), pm->priv->configure_button);
+  /* this box is needed to get the items at the end of the toolbar */
+  toolbar_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_container_add (GTK_CONTAINER (toolitem), toolbar_box);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
+
+  /* we need another box to disable css grouping */
+  item_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_pack_end (GTK_BOX (toolbar_box), item_box, FALSE, FALSE, 0);
+
+  pm->priv->about_button = gtk_button_new_with_mnemonic (_("_About"));
+  gtk_box_pack_start (GTK_BOX (item_box), pm->priv->about_button,
+                      FALSE, FALSE, 0);
+
+  /* we need another box to disable css grouping */
+  item_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_pack_end (GTK_BOX (toolbar_box), item_box, FALSE, FALSE, 0);
+
+  pm->priv->configure_button = gtk_button_new_with_mnemonic (_("_Preferences"));
+  gtk_box_pack_start (GTK_BOX (item_box), pm->priv->configure_button,
+                      FALSE, FALSE, 0);
 
   gtk_widget_pop_composite_child ();
 
@@ -457,10 +485,7 @@ peas_gtk_plugin_manager_constructed (GObject *object)
       g_warn_if_fail (engine == pm->priv->engine);
 
       if (engine != pm->priv->engine)
-        {
-          g_object_unref (pm->priv->view);
-          pm->priv->view = NULL;
-        }
+        g_clear_object (&pm->priv->view);
 
       g_object_unref (engine);
     }
@@ -498,8 +523,7 @@ peas_gtk_plugin_manager_constructed (GObject *object)
   /* Update the button sensitivity */
   selection_changed_cb (pm);
 
-  if (G_OBJECT_CLASS (peas_gtk_plugin_manager_parent_class)->constructed != NULL)
-    G_OBJECT_CLASS (peas_gtk_plugin_manager_parent_class)->constructed (object);
+  G_OBJECT_CLASS (peas_gtk_plugin_manager_parent_class)->constructed (object);
 }
 
 static void
@@ -512,9 +536,7 @@ peas_gtk_plugin_manager_dispose (GObject *object)
       g_signal_handlers_disconnect_by_func (pm->priv->engine,
                                             plugin_loaded_toggled_cb,
                                             pm);
-
-      g_object_unref (pm->priv->engine);
-      pm->priv->engine = NULL;
+      g_clear_object (&pm->priv->engine);
     }
 
   G_OBJECT_CLASS (peas_gtk_plugin_manager_parent_class)->dispose (object);
@@ -535,31 +557,30 @@ peas_gtk_plugin_manager_class_init (PeasGtkPluginManagerClass *klass)
    *
    * The #PeasEngine this manager is attached to.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_ENGINE,
-                                   g_param_spec_object ("engine",
-                                                        "engine",
-                                                        "The PeasEngine this manager is attached to",
-                                                        PEAS_TYPE_ENGINE,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
+  properties[PROP_ENGINE] =
+    g_param_spec_object ("engine",
+                         "engine",
+                         "The PeasEngine this manager is attached to",
+                         PEAS_TYPE_ENGINE,
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_STATIC_STRINGS);
 
   /**
    * PeasGtkPluginManager:view:
    *
    * The #PeasGtkPluginManagerView shown in the #PeasGtkPluginManager.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_VIEW,
-                                   g_param_spec_object ("view",
-                                                        "view",
-                                                        "The view shown in the manager",
-                                                        PEAS_GTK_TYPE_PLUGIN_MANAGER_VIEW,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
+  properties[PROP_VIEW] =
+    g_param_spec_object ("view",
+                         "view",
+                         "The view shown in the manager",
+                         PEAS_GTK_TYPE_PLUGIN_MANAGER_VIEW,
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_STATIC_STRINGS);
 
+  g_object_class_install_properties (object_class, N_PROPERTIES, properties);
   g_type_class_add_private (object_class, sizeof (PeasGtkPluginManagerPrivate));
 }
 
