@@ -216,6 +216,30 @@ peas_engine_rescan_plugins (PeasEngine *engine)
   g_object_thaw_notify (G_OBJECT (engine));
 }
 
+static void
+peas_engine_insert_search_path (PeasEngine  *engine,
+                                gint         position,
+                                const gchar *module_dir,
+                                const gchar *data_dir)
+{
+  SearchPath *sp;
+
+  g_return_if_fail (PEAS_IS_ENGINE (engine));
+  g_return_if_fail (module_dir != NULL);
+
+  sp = g_slice_new (SearchPath);
+  sp->module_dir = g_strdup (module_dir);
+  sp->data_dir = g_strdup (data_dir ? data_dir : module_dir);
+
+  engine->priv->search_paths = g_list_insert (engine->priv->search_paths,
+                                              sp,
+                                              position);
+
+  g_object_freeze_notify (G_OBJECT (engine));
+  load_dir_real (engine, sp->module_dir, sp->data_dir, 1);
+  g_object_thaw_notify (G_OBJECT (engine));
+}
+
 /**
  * peas_engine_add_search_path:
  * @engine: A #PeasEngine.
@@ -244,22 +268,26 @@ peas_engine_add_search_path (PeasEngine *engine,
                              const gchar *module_dir,
                              const gchar *data_dir)
 {
-  SearchPath *sp;
+  peas_engine_insert_search_path (engine, -1, module_dir, data_dir);
+}
 
-  g_return_if_fail (PEAS_IS_ENGINE (engine));
-  g_return_if_fail (module_dir != NULL);
-
-  sp = g_slice_new (SearchPath);
-  sp->module_dir = g_strdup (module_dir);
-  sp->data_dir = g_strdup (data_dir ? data_dir : module_dir);
-
-  /* Appending to a list is bad, but is easier to handle wrt refreshing
-   * the plugin list. */
-  engine->priv->search_paths = g_list_append (engine->priv->search_paths, sp);
-
-  g_object_freeze_notify (G_OBJECT (engine));
-  load_dir_real (engine, sp->module_dir, sp->data_dir, 1);
-  g_object_thaw_notify (G_OBJECT (engine));
+/**
+ * peas_engine_prepend_search_path:
+ * @engine: A #PeasEngine.
+ * @module_dir: the plugin module directory.
+ * @data_dir: (allow-none): the plugin data directory.
+ *
+ * This function prepends a search path to the list of paths where to
+ * look for plugins.
+ *
+ * See Also: peas_engine_add_search_path()
+ */
+void
+peas_engine_prepend_search_path (PeasEngine *engine,
+                                 const gchar *module_dir,
+                                 const gchar *data_dir)
+{
+  peas_engine_insert_search_path (engine, 0, module_dir, data_dir);
 }
 
 static guint
@@ -297,6 +325,17 @@ loader_destroy (LoaderInfo *info)
 static void
 peas_engine_init (PeasEngine *engine)
 {
+  /* Set the default engine here and not in constructor() to make sure
+   * that if a plugin is loaded and calls peas_engine_get_default()
+   * that this engine is returned and not another.
+   */
+  if (default_engine == NULL)
+    {
+      default_engine = engine;
+      g_object_add_weak_pointer (G_OBJECT (engine),
+                                 (gpointer *) &default_engine);
+    }
+
   engine->priv = G_TYPE_INSTANCE_GET_PRIVATE (engine,
                                               PEAS_TYPE_ENGINE,
                                               PeasEnginePrivate);
@@ -336,8 +375,6 @@ peas_engine_constructor (GType                  type,
                          guint                  n_construct_params,
                          GObjectConstructParam *construct_params)
 {
-  GObject *object;
-
   /* We don't support calling PeasEngine API without module support */
   if (!g_module_supported ())
     {
@@ -351,17 +388,9 @@ peas_engine_constructor (GType                  type,
                "as it has been shutdown.");
     }
 
-  object = G_OBJECT_CLASS (peas_engine_parent_class)->constructor (type,
-                                                                   n_construct_params,
-                                                                   construct_params);
-
-  if (default_engine == NULL)
-    {
-      default_engine = PEAS_ENGINE (object);
-      g_object_add_weak_pointer (object, (gpointer *) &default_engine);
-    }
-
-  return object;
+  return G_OBJECT_CLASS (peas_engine_parent_class)->constructor (type,
+                                                                 n_construct_params,
+                                                                 construct_params);
 }
 
 static void
@@ -1051,7 +1080,6 @@ peas_engine_create_extension_valist (PeasEngine     *engine,
                                      const gchar    *first_property,
                                      va_list         var_args)
 {
-  gpointer type_struct;
   guint n_parameters;
   GParameter *parameters;
   PeasExtension *exten;
@@ -1061,13 +1089,10 @@ peas_engine_create_extension_valist (PeasEngine     *engine,
   g_return_val_if_fail (peas_plugin_info_is_loaded (info), NULL);
   g_return_val_if_fail (G_TYPE_IS_INTERFACE (extension_type), FALSE);
 
-  type_struct = _g_type_struct_ref (extension_type);
-
-  if (!_valist_to_parameter_list (extension_type, type_struct, first_property,
+  if (!_valist_to_parameter_list (extension_type, first_property,
                                   var_args, &parameters, &n_parameters))
     {
       /* Already warned */
-      _g_type_struct_unref (extension_type, type_struct);
       return NULL;
     }
 
@@ -1078,8 +1103,6 @@ peas_engine_create_extension_valist (PeasEngine     *engine,
     g_value_unset (&parameters[n_parameters].value);
   g_free (parameters);
 
-  _g_type_struct_unref (extension_type, type_struct);
-
   return exten;
 }
 
